@@ -388,8 +388,11 @@ on_evdev_event (ManetteDevice      *self,
     manette_event.any.type = evdev_event->value ?
       MANETTE_EVENT_BUTTON_PRESS :
       MANETTE_EVENT_BUTTON_RELEASE;
-    manette_event.button.hardware_index =
-      self->key_map[evdev_event->code - BTN_MISC];
+    guint index = evdev_event->code < BTN_MISC ?
+      evdev_event->code + BTN_MISC :
+      evdev_event->code - BTN_MISC;
+
+    manette_event.button.hardware_index = self->key_map[index];
     manette_event.button.button = evdev_event->code;
 
     break;
@@ -530,6 +533,11 @@ manette_device_new (const gchar  *filename,
       self->key_map[i - BTN_MISC] = (guint8) buttons_number;
       buttons_number++;
     }
+  for (i = 0; i < BTN_MISC; i++)
+    if (has_key (self->evdev_device, i)) {
+      self->key_map[i + BTN_MISC] = (guint8) buttons_number;
+      buttons_number++;
+    }
 
   // Get info about the axes.
   axes_number = 0;
@@ -563,6 +571,8 @@ manette_device_new (const gchar  *filename,
  * devices.
  *
  * Returns: (transfer none): the identifier used by SDL mappings
+ *
+ * Since: 0.3
  */
 const gchar *
 manette_device_get_guid (ManetteDevice *self)
@@ -701,6 +711,31 @@ manette_device_set_mapping (ManetteDevice  *self,
   g_set_object (&self->mapping, mapping);
 }
 
+/**
+ * manette_device_get_mapping:
+ * @self: a #ManetteDevice
+ *
+ * Gets the user mapping for @self, or default mapping if there isn't any. Can
+ * return %NULL if there's no mapping.
+ *
+ * Returns: (transfer full) (nullable): the mapping for @self
+ *
+ * Since: 0.3
+ */
+char *
+manette_device_get_mapping (ManetteDevice *self)
+{
+  const gchar *guid;
+  g_autoptr (ManetteMappingManager) mapping_manager = NULL;
+
+  g_return_val_if_fail (MANETTE_IS_DEVICE (self), FALSE);
+
+  guid = manette_device_get_guid (self);
+  mapping_manager = manette_mapping_manager_new ();
+
+  return manette_mapping_manager_get_mapping (mapping_manager, guid);
+}
+
 /**
  * manette_device_has_user_mapping:
  * @self: a #ManetteDevice
