@@ -58,6 +58,7 @@ add_mapping (ManetteMappingManager *self,
              GHashTable            *mappings)
 {
   const gchar *platform;
+  const gchar *hint;
   g_auto (GStrv) split = NULL;
 
   g_assert (mapping_string != NULL);
@@ -66,8 +67,16 @@ add_mapping (ManetteMappingManager *self,
     return;
 
   platform = g_strstr_len (mapping_string, -1, "platform");
-  if (platform != NULL && !g_str_has_prefix (platform, "platform:Linux"))
+  if (platform != NULL && !g_str_has_prefix (platform, "platform:Linux")) {
+    g_debug ("Mappings for other platforms than Linux aren´t supported: ignoring mapping `%s`", mapping_string);
     return;
+  }
+
+  hint = g_strstr_len (mapping_string, -1, "hint");
+  if (hint != NULL && !g_str_has_prefix (hint, "hint:SDL_GAMECONTROLLER_USE_BUTTON_LABELS:=1")) {
+    g_debug ("Mappings reporting face buttons by label instead of position aren´t supported: ignoring mapping `%s`", mapping_string);
+    return;
+  }
 
   /* GUID | device name | the rest of the mapping string */
   split = g_strsplit (mapping_string, ",", 3);
@@ -197,6 +206,7 @@ user_mappings_changed_cb (GFileMonitor          *monitor,
                           GFileMonitorEvent      event_type,
                           ManetteMappingManager *self)
 {
+  g_autoptr (GFile) user_mappings_file = NULL;
   g_autoptr (GError) error = NULL;
 
   g_hash_table_remove_all (self->user_mappings);
@@ -207,7 +217,9 @@ user_mappings_changed_cb (GFileMonitor          *monitor,
     return;
   }
 
-  add_from_file_uri (self, self->user_mappings_uri, self->user_mappings, &error);
+  user_mappings_file = g_file_new_for_uri (self->user_mappings_uri);
+  if (g_file_query_exists (user_mappings_file, NULL))
+    add_from_file_uri (self, self->user_mappings_uri, self->user_mappings, &error);
   if (G_UNLIKELY (error != NULL)) {
     g_debug ("ManetteMappingManager: Can’t add mappings from %s: %s",
              self->user_mappings_uri,
@@ -273,7 +285,8 @@ manette_mapping_manager_new (void)
                     G_CALLBACK (user_mappings_changed_cb),
                     self);
 
-  add_from_file_uri (self, self->user_mappings_uri, self->user_mappings, &error);
+  if (g_file_query_exists (user_mappings_file, NULL))
+    add_from_file_uri (self, self->user_mappings_uri, self->user_mappings, &error);
   if (G_UNLIKELY (error != NULL)) {
     g_debug ("ManetteMappingManager: Can’t add mappings from %s: %s",
              self->user_mappings_uri,
@@ -376,6 +389,14 @@ manette_mapping_manager_delete_mapping (ManetteMappingManager *self,
     g_critical ("ManetteMappingManager: Can’t save user mappings: %s", error->message);
 }
 
+GList *
+manette_mapping_manager_get_default_mappings (ManetteMappingManager *self)
+{
+  g_return_val_if_fail (MANETTE_IS_MAPPING_MANAGER (self), NULL);
+
+  return g_hash_table_get_values (self->default_mappings);
+}
+
 /* Type */
 
 static void
