@@ -12,7 +12,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -35,6 +35,27 @@ G_DEFINE_TYPE (ManetteMapping, manette_mapping, G_TYPE_OBJECT)
 
 G_DEFINE_BOXED_TYPE (ManetteMappingBinding, manette_mapping_binding, manette_mapping_binding_copy, manette_mapping_binding_free)
 
+#ifdef G_DISABLE_CHECKS
+
+#define manette_ensure_is_parseable(expr) G_STMT_START{ (void)0; }G_STMT_END
+
+#else
+
+#define manette_ensure_is_parseable(expr) \
+  G_STMT_START { \
+    if (G_LIKELY (expr != NULL)) { } \
+    else  { \
+      g_log (G_LOG_DOMAIN, \
+             G_LOG_LEVEL_DEBUG, \
+             "%s: expression '%s' not parseable", \
+             G_STRFUNC, \
+             #expr); \
+      return FALSE; \
+    } \
+  } G_STMT_END
+
+#endif
+
 /* Private */
 
 static void
@@ -42,12 +63,9 @@ manette_mapping_finalize (GObject *object)
 {
   ManetteMapping *self = (ManetteMapping *)object;
 
-  if (self->axis_bindings != NULL)
-    g_array_free (self->axis_bindings, TRUE);
-  if (self->button_bindings != NULL)
-    g_array_free (self->button_bindings, TRUE);
-  if (self->hat_bindings != NULL)
-    g_array_free (self->hat_bindings, TRUE);
+  g_clear_pointer (&self->axis_bindings, g_array_unref);
+  g_clear_pointer (&self->button_bindings, g_array_unref);
+  g_clear_pointer (&self->hat_bindings, g_array_unref);
 
   G_OBJECT_CLASS (manette_mapping_parent_class)->finalize (object);
 }
@@ -99,9 +117,9 @@ try_str_to_guint16 (gchar    *start,
                     gchar   **end,
                     guint16  *result)
 {
-  g_return_val_if_fail (start != NULL, FALSE);
-  g_return_val_if_fail (end != NULL, FALSE);
-  g_return_val_if_fail (result != NULL, FALSE);
+  manette_ensure_is_parseable (start);
+  manette_ensure_is_parseable (end);
+  manette_ensure_is_parseable (result);
 
   errno = 0;
   *result = strtol (start, end, 10);
@@ -114,9 +132,9 @@ parse_mapping_input_type (gchar                    *start,
                           gchar                   **end,
                           ManetteMappingInputType  *input_type)
 {
-  g_return_val_if_fail (start != NULL, FALSE);
-  g_return_val_if_fail (end != NULL, FALSE);
-  g_return_val_if_fail (input_type != NULL, FALSE);
+  manette_ensure_is_parseable (start);
+  manette_ensure_is_parseable (end);
+  manette_ensure_is_parseable (input_type);
 
   switch (*start) {
   case 'a':
@@ -152,9 +170,9 @@ parse_mapping_invert (gchar     *start,
                       gchar    **end,
                       gboolean  *invert)
 {
-  g_return_val_if_fail (start != NULL, FALSE);
-  g_return_val_if_fail (end != NULL, FALSE);
-  g_return_val_if_fail (invert != NULL, FALSE);
+  manette_ensure_is_parseable (start);
+  manette_ensure_is_parseable (end);
+  manette_ensure_is_parseable (invert);
 
   switch (*start) {
   case '~':
@@ -174,9 +192,9 @@ parse_mapping_range (gchar                *start,
                      gchar               **end,
                      ManetteMappingRange  *range)
 {
-  g_return_val_if_fail (start != NULL, FALSE);
-  g_return_val_if_fail (end != NULL, FALSE);
-  g_return_val_if_fail (range != NULL, FALSE);
+  manette_ensure_is_parseable (start);
+  manette_ensure_is_parseable (end);
+  manette_ensure_is_parseable (range);
 
   switch (*start) {
   case '+':
@@ -208,11 +226,11 @@ parse_mapping_hat (gchar                *start,
   guint16 hat_position_2pow;
   guint16 hat_position = 0;
 
-  g_return_val_if_fail (start != NULL, FALSE);
-  g_return_val_if_fail (end != NULL, FALSE);
-  g_return_val_if_fail (index != NULL, FALSE);
-  g_return_val_if_fail (range != NULL, FALSE);
-  g_return_val_if_fail (invert != NULL, FALSE);
+  manette_ensure_is_parseable (start);
+  manette_ensure_is_parseable (end);
+  manette_ensure_is_parseable (index);
+  manette_ensure_is_parseable (range);
+  manette_ensure_is_parseable (invert);
 
   if (!try_str_to_guint16 (start, end, &hat_index))
     return FALSE;
@@ -227,7 +245,7 @@ parse_mapping_hat (gchar                *start,
   if (!try_str_to_guint16 (start, end, &hat_position_2pow))
     return FALSE;
 
-  // hat_position: 0 up, 1 right, 2 down, 3 left.
+  /* hat_position: 0 up, 1 right, 2 down, 3 left. */
   while (hat_position_2pow > 1) {
     hat_position_2pow >>= 1;
     hat_position++;
@@ -236,9 +254,9 @@ parse_mapping_hat (gchar                *start,
   if (hat_position > 3)
     return FALSE;
 
-  // hat0x, hat0y, hat1x, hat1y…
+  /* hat0x, hat0y, hat1x, hat1y… */
   *index = hat_index * 2 + (hat_position + 1) % 2;
-  // left or up: negative, right or down: position.
+  /* left or up: negative, right or down: positive. */
   *range = ((hat_position + 1) % 4) < 2 ? MANETTE_MAPPING_RANGE_NEGATIVE :
                                           MANETTE_MAPPING_RANGE_POSITIVE;
   *invert = *range == MANETTE_MAPPING_RANGE_NEGATIVE;
@@ -278,6 +296,17 @@ parse_destination_input (gchar    *start,
     { EV_KEY, BTN_TR2, "righttrigger" },
     { EV_KEY, BTN_Y, "x" },
     { EV_KEY, BTN_X, "y" },
+    { EV_KEY, BTN_TRIGGER_HAPPY7, "paddle1" },
+    { EV_KEY, BTN_TRIGGER_HAPPY8, "paddle2" },
+    { EV_KEY, BTN_TRIGGER_HAPPY9, "paddle3" },
+    { EV_KEY, BTN_TRIGGER_HAPPY10, "paddle4" },
+    { EV_KEY, BTN_TRIGGER_HAPPY11, "touchpad" },
+    { EV_KEY, BTN_TRIGGER_HAPPY1, "misc1" },
+    { EV_KEY, BTN_TRIGGER_HAPPY2, "misc2" },
+    { EV_KEY, BTN_TRIGGER_HAPPY3, "misc3" },
+    { EV_KEY, BTN_TRIGGER_HAPPY4, "misc4" },
+    { EV_KEY, BTN_TRIGGER_HAPPY5, "misc5" },
+    { EV_KEY, BTN_TRIGGER_HAPPY6, "misc6" },
   };
   const gint length = sizeof (values) / sizeof (values[0]);
   gint i;
@@ -297,13 +326,8 @@ parse_destination_input (gchar    *start,
 static void
 manette_mapping_binding_try_free (ManetteMappingBinding **binding)
 {
-  if (binding == NULL)
-    return;
-
-  if (*binding == NULL)
-    return;
-
-  manette_mapping_binding_free (*binding);
+  if (G_LIKELY (binding))
+    g_clear_pointer (binding, manette_mapping_binding_free);
 }
 
 static void
@@ -411,59 +435,81 @@ parse_mapping_source (gchar                 *source,
   return TRUE;
 }
 
-// This function doesn't take care of cleaning up the object's state before
-// setting it.
-static void
-set_from_mapping_string (ManetteMapping *self,
-                         const gchar    *mapping_string)
+static gboolean
+is_valid_guid (const gchar *string)
 {
-  gchar **mappings;
-  guint mappings_length;
+  if (!string)
+    return FALSE;
+
+  for (guint i = 0; i < 32; i++)
+    if (!g_ascii_isxdigit (string[i]))
+      return FALSE;
+
+  return TRUE;
+}
+
+/* This function doesn't take care of cleaning up the object's state before
+ * setting it.
+ */
+static void
+set_from_mapping_string (ManetteMapping  *self,
+                         const gchar     *mapping_string,
+                         GError         **error)
+{
+  g_auto(GStrv) mappings = g_strsplit (mapping_string, ",", 0);
+  guint mappings_length = g_strv_length (mappings);
   guint i = 0;
-  gchar **splitted_mapping;
   gchar *destination_string;
   gchar *source_string;
   ManetteMappingBinding binding = {};
 
-  mappings = g_strsplit (mapping_string, ",", 0);
-  mappings_length = g_strv_length (mappings);
-  for (i = 0; i < mappings_length; i++) {
+  if (mappings_length < 2) {
+    g_set_error (error,
+                 MANETTE_MAPPING_ERROR,
+                 MANETTE_MAPPING_ERROR_NOT_A_MAPPING,
+                 "Invalid mapping string: %s",
+                 mapping_string);
 
-    splitted_mapping = g_strsplit (mappings[i], ":", 0);
+    return;
+  }
 
-    if (g_strv_length (splitted_mapping) != 2) {
-      g_strfreev (splitted_mapping);
+  if (!is_valid_guid (mappings[0])) {
+    g_set_error (error,
+                 MANETTE_MAPPING_ERROR,
+                 MANETTE_MAPPING_ERROR_NOT_A_MAPPING,
+                 "Invalid mapping string: no GUID: %s",
+                 mapping_string);
 
+    return;
+  }
+
+  for (i = 2; i < mappings_length; i++) {
+    g_auto(GStrv) splitted_mapping = g_strsplit (mappings[i], ":", 0);
+
+    if (g_strv_length (splitted_mapping) != 2)
       continue;
-    }
 
     destination_string = splitted_mapping[0];
     source_string = splitted_mapping[1];
 
     /* Skip the platform key. */
-    if (g_strcmp0 ("platform", splitted_mapping[0]) == 0) {
-      g_strfreev (splitted_mapping);
-
+    if (g_strcmp0 ("platform", splitted_mapping[0]) == 0)
       continue;
-    }
 
     if (!parse_mapping_destination (destination_string, &binding)) {
-      g_critical ("Invalid mapping destination: %s", splitted_mapping[0]);
-      g_strfreev (splitted_mapping);
+      g_critical ("Invalid binding destination: %s:%s in %s", destination_string, source_string, mapping_string);
 
       continue;
     }
 
     if  (binding.destination.type == EV_MAX) {
-      g_debug ("Invalid token: %s", destination_string);
-      g_strfreev (splitted_mapping);
+      g_critical ("Invalid token: %s", destination_string);
 
       continue;
     }
 
     if (!parse_mapping_source (source_string, &binding)) {
-      g_critical ("Invalid mapping source: %s", splitted_mapping[1]);
-      g_strfreev (splitted_mapping);
+      g_critical ("Invalid binding source: %s:%s in %s", destination_string, source_string, mapping_string);
 
       continue;
     }
@@ -484,30 +530,22 @@ set_from_mapping_string (ManetteMapping *self,
     default:
       g_assert_not_reached ();
     }
-
-    g_strfreev (splitted_mapping);
   }
-
-  g_strfreev (mappings);
 }
 
 static void
 g_array_try_free (GArray **array)
 {
-  if (array == NULL)
-    return;
-
-  if (*array == NULL)
-    return;
-
-  g_array_free (*array, TRUE);
+  if (G_LIKELY (array))
+    g_clear_pointer (array, g_array_unref);
 }
 
 ManetteMapping *
 manette_mapping_new (const gchar  *mapping_string,
                      GError      **error)
 {
-  ManetteMapping *self = NULL;
+  g_autoptr (ManetteMapping) self = NULL;
+  GError *inner_error = NULL;
 
   if (mapping_string == NULL) {
     g_set_error_literal (error,
@@ -536,9 +574,14 @@ manette_mapping_new (const gchar  *mapping_string,
   self->hat_bindings = g_array_new (FALSE, TRUE, sizeof (GArray *));
   g_array_set_clear_func (self->hat_bindings, (GDestroyNotify) g_array_try_free);
 
-  set_from_mapping_string (self, mapping_string);
+  set_from_mapping_string (self, mapping_string, &inner_error);
+  if (G_UNLIKELY (inner_error != NULL)) {
+    g_propagate_error (error, inner_error);
 
-  return self;
+    return NULL;
+  }
+
+  return g_steal_pointer (&self);
 }
 
 const ManetteMappingBinding * const *
