@@ -12,26 +12,30 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * SECTION:manette-monitor
+ * @short_description: An object monitoring the availability of devices
+ * @title: ManetteMonitor
+ * @See_also: #ManetteDevice
+ */
+
 #include "manette-monitor.h"
 
 #include <glib.h>
 #include <glib-object.h>
+#include <gio/gio.h>
 #ifdef GUDEV_ENABLED
  #include <gudev/gudev.h>
-#else
- #include <gio/gio.h>
 #endif
 #include "manette-device-private.h"
 #include "manette-mapping-manager-private.h"
 #include "manette-monitor-iter-private.h"
 
-#ifndef GUDEV_ENABLED
 #define INPUT_DIRECTORY "/dev/input"
-#endif
 
 struct _ManetteMonitor {
   GObject parent_instance;
@@ -40,10 +44,9 @@ struct _ManetteMonitor {
   ManetteMappingManager *mapping_manager;
 #ifdef GUDEV_ENABLED
   GUdevClient *client;
-#else
+#endif
   GFileMonitor *monitor;
   GHashTable *potential_devices;
-#endif
 };
 
 G_DEFINE_TYPE (ManetteMonitor, manette_monitor, G_TYPE_OBJECT)
@@ -58,19 +61,22 @@ static guint signals[N_SIGNALS];
 
 /* Private */
 
-static void
-manette_monitor_init (ManetteMonitor *self)
+#if GUDEV_ENABLED
+static inline gboolean
+is_flatpak (void)
 {
+  return g_file_test ("/.flatpak-info", G_FILE_TEST_EXISTS);
 }
+#endif
 
 static void
 load_mapping (ManetteMonitor *self,
               ManetteDevice  *device)
 {
   const gchar *guid;
-  gchar *mapping_string;
-  ManetteMapping *mapping = NULL;
-  GError *error = NULL;
+  g_autofree gchar *mapping_string = NULL;
+  g_autoptr (ManetteMapping) mapping = NULL;
+  g_autoptr (GError) error = NULL;
 
   guid = manette_device_get_guid (device);
   mapping_string = manette_mapping_manager_get_mapping (self->mapping_manager,
@@ -78,27 +84,22 @@ load_mapping (ManetteMonitor *self,
   mapping = manette_mapping_new (mapping_string, &error);
   if (G_UNLIKELY (error != NULL)) {
     g_debug ("%s", error->message);
-    g_clear_error (&error);
+
+    return;
   }
 
-  if (mapping_string != NULL)
-    g_free (mapping_string);
-
   manette_device_set_mapping (device, mapping);
-
-  if (mapping != NULL)
-    g_object_unref (mapping);
 }
 
 static void
 add_device (ManetteMonitor *self,
             const gchar    *filename)
 {
-  ManetteDevice *device;
-  GError *error = NULL;
+  g_autoptr (ManetteDevice) device = NULL;
+  g_autoptr (GError) error = NULL;
 
-  g_return_if_fail (self != NULL);
-  g_return_if_fail (filename != NULL);
+  g_assert (self != NULL);
+  g_assert (filename != NULL);
 
   if (g_hash_table_contains (self->devices, filename))
     return;
@@ -108,9 +109,6 @@ add_device (ManetteMonitor *self,
     if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NXIO))
       g_debug ("Failed to open %s: %s", filename, error->message);
 
-    g_error_free (error);
-    error = NULL;
-
     return;
   }
 
@@ -120,7 +118,6 @@ add_device (ManetteMonitor *self,
                        g_strdup (filename),
                        g_object_ref (device));
   g_signal_emit (self, signals[SIG_DEVICE_CONNECTED], 0, device);
-  g_object_unref (device);
 }
 
 static void
@@ -140,7 +137,7 @@ remove_device (ManetteMonitor *self,
   g_object_unref (device);
 }
 
-#ifdef GUDEV_ENABLED /* BACKEND GUDEV */
+#ifdef GUDEV_ENABLED
 
 static void
 add_device_for_udev_device (ManetteMonitor *self,
@@ -148,8 +145,8 @@ add_device_for_udev_device (ManetteMonitor *self,
 {
   const gchar *filename;
 
-  g_return_if_fail (self != NULL);
-  g_return_if_fail (udev_device != NULL);
+  g_assert (self != NULL);
+  g_assert (udev_device != NULL);
 
   filename = g_udev_device_get_device_file (udev_device);
   add_device (self, filename);
@@ -170,32 +167,30 @@ udev_device_property_is (GUdevDevice *udev_device,
                          const gchar *property,
                          const gchar *value)
 {
-  return g_udev_device_has_property (udev_device, property) &&
-         (g_strcmp0 (g_udev_device_get_property (udev_device, property), value) == 0);
+  g_assert (property != NULL);
+  g_assert (value != NULL);
+
+  return g_strcmp0 (g_udev_device_get_property (udev_device, property), value) == 0;
 }
 
 static gboolean
 udev_device_is_manette (GUdevDevice *udev_device)
 {
-  g_return_val_if_fail (udev_device != NULL, FALSE);
+  g_assert (udev_device != NULL);
 
   return udev_device_property_is (udev_device, "ID_INPUT_JOYSTICK", "1") ||
          udev_device_property_is (udev_device, ".INPUT_CLASS", "joystick");
 }
 
 static void
-handle_udev_client_callback (GUdevClient *sender,
-                             const gchar *action,
-                             GUdevDevice *udev_device,
-                             gpointer     data)
+udev_client_uevent_cb (GUdevClient    *sender,
+                       const gchar    *action,
+                       GUdevDevice    *udev_device,
+                       ManetteMonitor *self)
 {
-  ManetteMonitor *self;
-
-  self = MANETTE_MONITOR (data);
-
-  g_return_if_fail (self != NULL);
-  g_return_if_fail (action != NULL);
-  g_return_if_fail (udev_device != NULL);
+  g_assert (action != NULL);
+  g_assert (udev_device != NULL);
+  g_assert (self != NULL);
 
   if (g_udev_device_get_device_file (udev_device) == NULL)
     return;
@@ -210,7 +205,7 @@ handle_udev_client_callback (GUdevClient *sender,
 }
 
 static void
-coldplug_devices (ManetteMonitor *self)
+coldplug_gudev_devices (ManetteMonitor *self)
 {
   GList *initial_devices_list;
   GList *device_it = NULL;
@@ -236,17 +231,30 @@ coldplug_devices (ManetteMonitor *self)
 }
 
 static void
-init_backend (ManetteMonitor *self)
+init_gudev_backend (ManetteMonitor *self)
 {
   self->client = g_udev_client_new ((const gchar *[]) { "input", NULL });
   g_signal_connect_object (self->client,
                            "uevent",
-                           (GCallback) handle_udev_client_callback,
+                           (GCallback) udev_client_uevent_cb,
                            self,
                            0);
 }
 
-#else /* BACKEND FALLBACK */
+#endif
+
+/* This eliminates all other files that can be in /dev/input, like js*, mouse*,
+ * by-id/, etc.
+ * There isn't really any need to check if there's only digits after "event", as
+ * it would induce a bit of performance loss for this hypothetical case…
+ */
+static gboolean
+is_evdev_device (GFile *file)
+{
+  g_autofree char *event_file_basename = g_file_get_basename (file);
+
+  return g_str_has_prefix (event_file_basename, "event");
+}
 
 static gboolean
 is_accessible (GFile *file)
@@ -309,12 +317,15 @@ file_deleted (ManetteMonitor *self,
 }
 
 static void
-handle_file_monitor_callback (GFileMonitor      *monitor,
-                              GFile             *file,
-                              GFile             *other_file,
-                              GFileMonitorEvent  event_type,
-                              ManetteMonitor    *self)
+file_monitor_changed_cb (GFileMonitor      *monitor,
+                         GFile             *file,
+                         GFile             *other_file,
+                         GFileMonitorEvent  event_type,
+                         ManetteMonitor    *self)
 {
+  if (!is_evdev_device (file))
+    return;
+
   switch (event_type) {
   case G_FILE_MONITOR_EVENT_CREATED:
     file_created (self, file);
@@ -334,32 +345,32 @@ handle_file_monitor_callback (GFileMonitor      *monitor,
 }
 
 static void
-coldplug_devices (ManetteMonitor *self)
+coldplug_file_devices (ManetteMonitor *self)
 {
-  GDir *dir;
+  g_autoptr (GDir) dir = NULL;
   const gchar *name = NULL;
-  gchar *filename;
-  GError *error = NULL;
+  g_autoptr (GError) error = NULL;
 
   dir = g_dir_open (INPUT_DIRECTORY, (guint) 0, &error);
   if (G_UNLIKELY (error != NULL)) {
     g_debug ("%s", error->message);
-    g_error_free (error);
 
     return;
   }
 
   while ((name = g_dir_read_name (dir)) != NULL) {
-    filename = g_build_filename (INPUT_DIRECTORY, name, NULL);
-    add_device (self, filename);
-    g_free (filename);
-  }
+    g_autofree gchar *filename = NULL;
+    g_autoptr (GFile) file = NULL;
 
-  g_dir_close (dir);
+    filename = g_build_filename (INPUT_DIRECTORY, name, NULL);
+    file = g_file_new_for_path (filename);
+    if (is_evdev_device (file) && is_accessible (file))
+      add_device (self, filename);
+  }
 }
 
 static void
-init_backend (ManetteMonitor *self)
+init_file_backend (ManetteMonitor *self)
 {
   g_autoptr (GFile) file = g_file_new_for_path (INPUT_DIRECTORY);
   g_autoptr (GError) error = NULL;
@@ -374,7 +385,7 @@ init_backend (ManetteMonitor *self)
   else
     g_signal_connect_object (self->monitor,
                              "changed",
-                             (GCallback) handle_file_monitor_callback,
+                             (GCallback) file_monitor_changed_cb,
                              self,
                              0);
 
@@ -382,63 +393,80 @@ init_backend (ManetteMonitor *self)
                                                    g_free, NULL);
 }
 
-#endif /* BACKEND */
-
 static void
-on_mappings_changed (ManetteMappingManager *mapping_manager,
+mappings_changed_cb (ManetteMappingManager *mapping_manager,
                      ManetteMonitor        *self)
 {
-  ManetteMonitorIter *iterator;
+  g_autoptr (ManetteMonitorIter) iterator = NULL;
   ManetteDevice *device = NULL;
 
-  g_return_if_fail (MANETTE_IS_MONITOR (self));
-
   iterator = manette_monitor_iterate (self);
   while (manette_monitor_iter_next (iterator, &device))
     load_mapping (self, device);
-  manette_monitor_iter_free (iterator);
 }
 
 /* Public */
 
+/**
+ * manette_monitor_new:
+ *
+ * Creates a new #ManetteMonitor object.
+ *
+ * Returns: (transfer full): a new #ManetteMonitor
+ */
 ManetteMonitor *
 manette_monitor_new (void)
 {
-  ManetteMonitor *self = NULL;
-
-  self = (ManetteMonitor*) g_object_new (MANETTE_TYPE_MONITOR, NULL);
-  self->devices = g_hash_table_new_full (g_str_hash, g_str_equal,
-                                         g_free, g_object_unref);
-  self->mapping_manager = manette_mapping_manager_new ();
-
-  g_signal_connect (self->mapping_manager,
-                    "changed",
-                    G_CALLBACK (on_mappings_changed),
-                    self);
-
-  init_backend (self);
-  coldplug_devices (self);
-
-  return self;
+  return g_object_new (MANETTE_TYPE_MONITOR, NULL);
 }
 
 /* Type */
 
 static void
-finalize (GObject *object)
+manette_monitor_init (ManetteMonitor *self)
+{
+  gboolean use_file_backend = FALSE;
+
+  self->devices = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                         g_free, g_object_unref);
+  self->mapping_manager = manette_mapping_manager_new ();
+
+  g_signal_connect_object (self->mapping_manager,
+                           "changed",
+                           G_CALLBACK (mappings_changed_cb),
+                           self, 0);
+
+#if GUDEV_ENABLED
+  use_file_backend = is_flatpak ();
+#else
+  use_file_backend = TRUE;
+#endif
+
+  if (use_file_backend) {
+    init_file_backend (self);
+    coldplug_file_devices (self);
+  } else {
+#if GUDEV_ENABLED
+    init_gudev_backend (self);
+    coldplug_gudev_devices (self);
+#endif
+  }
+}
+
+static void
+manette_monitor_finalize (GObject *object)
 {
   ManetteMonitor *self = MANETTE_MONITOR (object);
 
 #ifdef GUDEV_ENABLED
-  if (self->client != NULL)
-    g_object_unref (self->client);
-#else
-  g_clear_object (&self->monitor);
-  g_hash_table_unref (self->potential_devices);
+  g_clear_object (&self->client);
 #endif
 
-  g_object_unref (self->mapping_manager);
-  g_hash_table_unref (self->devices);
+  g_clear_object (&self->monitor);
+  g_clear_pointer (&self->potential_devices, g_hash_table_unref);
+
+  g_clear_object (&self->mapping_manager);
+  g_clear_pointer (&self->devices, g_hash_table_unref);
 
   G_OBJECT_CLASS (manette_monitor_parent_class)->finalize (object);
 }
@@ -447,7 +475,9 @@ static void
 manette_monitor_class_init (ManetteMonitorClass *klass)
 {
   manette_monitor_parent_class = g_type_class_peek_parent (klass);
-  G_OBJECT_CLASS (klass)->finalize = finalize;
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  object_class->finalize = manette_monitor_finalize;
 
   /**
    * ManetteMonitor::device-connected:
@@ -486,7 +516,9 @@ manette_monitor_class_init (ManetteMonitorClass *klass)
  * manette_monitor_iterate:
  * @self: a #ManetteMonitor
  *
- * Returns: (transfer full): a new #ManetteMonitorIter
+ * Creates a new #ManetteMonitorIter iterating on @self.
+ *
+ * Returns: (transfer full): a new #ManetteMonitorIter iterating on @self
  */
 ManetteMonitorIter *
 manette_monitor_iterate (ManetteMonitor *self)
